# Copyright © 2025 GlacieTeam.All rights reserved.
#
# This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy
# of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# SPDX-License-Identifier: MPL-2.0

import os
from typing import Union, Optional
from rapidnbt._NBT.compound_tag import CompoundTag
from rapidnbt._NBT.snbt_format import SnbtFormat
from rapidnbt._NBT.nbt_file_format import NbtFileFormat
from rapidnbt._NBT.nbt_compression_level import NbtCompressionLevel
from rapidnbt._NBT.nbt_compression_type import NbtCompressionType
from rapidnbt._NBT.nbt_file import NbtFile

def detect_content_format(
    content: Union[bytes, bytearray], strict_match_size: bool = True
) -> Optional[NbtFileFormat]:
    """
    Detect NBT format from binary content

    Args:
        content (bytes): Binary content to analyze
        strict_match_size (bool): Strictly match nbt content size (default: True)

    Returns:
        NbtFileFormat or None if format cannot be determined

    """

def detect_file_format(
    path: os.PathLike, file_memory_map: bool = False, strict_match_size: bool = True
) -> Optional[NbtFileFormat]:
    """
    Detect NBT format from a file

    Args:
        path (os.PathLike): Path to the file
        file_memory_map (bool): Use memory mapping for large files (default: False)
        strict_match_size (bool): Strictly match nbt content size (default: True)

    Returns:
        NbtFileFormat or None if format cannot be determined

    """

def detect_content_compression_type(
    content: Union[bytes, bytearray],
) -> NbtCompressionType:
    """
    Detect NBT format from binary content

    Args:
        content (bytes): Binary content to analyze

    Returns:
        NbtCompressionType

    """

def detect_file_compression_type(
    path: os.PathLike, file_memory_map: bool = False
) -> NbtCompressionType:
    """
    Detect NBT format from a file

    Args:
        path (os.PathLike): Path to the file
        file_memory_map (bool): Use memory mapping for large files (default: False)

    Returns:
        NbtCompressionType

    """

def dump(
    nbt: CompoundTag,
    path: os.PathLike,
    format: NbtFileFormat = NbtFileFormat.LITTLE_ENDIAN,
    compression_type: NbtCompressionType = NbtCompressionType.GZIP,
    compression_level: NbtCompressionLevel = NbtCompressionLevel.DEFAULT,
) -> bool:
    """
    Save CompoundTag to a file

    Args:
        nbt (CompoundTag): Tag to save
        path (os.PathLike): Output file path
        format (NbtFileFormat): Output format (default: LITTLE_ENDIAN)
        compression_type (CompressionType): Compression method (default: Gzip)
        compression_level (CompressionLevel): Compression level (default: Default)

    Returns:
        bool: True if successful, False otherwise

    """

def dump_snbt(
    nbt: CompoundTag,
    path: os.PathLike,
    format: SnbtFormat = SnbtFormat.Default,
    indent: int = 4,
) -> bool:
    """
    Save CompoundTag to SNBT (String NBT) file

    Args:
        nbt (CompoundTag): Tag to save
        path (os.PathLike): Output file path
        format (SnbtFormat): Output formatting style (default: Default)
        indent (int): Indentation level (default: 4)

    Returns:
        bool: True if successful, False otherwise

    """

def dumps(
    nbt: CompoundTag,
    format: NbtFileFormat = NbtFileFormat.LITTLE_ENDIAN,
    compression_type: NbtCompressionType = NbtCompressionType.GZIP,
    compression_level: NbtCompressionLevel = NbtCompressionLevel.DEFAULT,
) -> bytes:
    """
    Serialize CompoundTag to binary data

    Args:
        nbt (CompoundTag): Tag to serialize
        format (NbtFileFormat): Output format (default: LITTLE_ENDIAN)
        compression_type (CompressionType): Compression method (default: Gzip)
        compression_level (CompressionLevel): Compression level (default: Default)

    Returns:
        bytes: Serialized binary data

    """

def dumps_snbt(
    nbt: CompoundTag,
    format: SnbtFormat = SnbtFormat.Default,
    indent: int = 4,
) -> str:
    """
    Save CompoundTag to SNBT (String NBT) file

    Args:
        nbt (CompoundTag): Tag to save
        format (SnbtFormat): Output formatting style (default: Default)
        indent (int): Indentation level (default: 4)

    Returns:
        bool: SNBT string

    """

def dumps_base64(
    nbt: CompoundTag,
    format: NbtFileFormat = NbtFileFormat.LITTLE_ENDIAN,
    compression_type: NbtCompressionType = NbtCompressionType.GZIP,
    compression_level: NbtCompressionLevel = NbtCompressionLevel.DEFAULT,
) -> str:
    """
    Serialize CompoundTag to Base64 string

    Args:
        nbt (CompoundTag): Tag to serialize
        format (NbtFileFormat): Output format (default: LITTLE_ENDIAN)
        compression_type (CompressionType): Compression method (default: Gzip)
        compression_level (CompressionLevel): Compression level (default: Default)

    Returns:
        str: Base64-encoded NBT data

    """

def load(
    path: os.PathLike,
    format: Optional[NbtFileFormat] = None,
    file_memory_map: bool = False,
    strict_match_size: bool = True,
) -> Optional[CompoundTag]:
    """
    Parse CompoundTag from a file

    Args:
        path (os.PathLike): Path to NBT file
        format (NbtFileFormat, optional): Force specific format (autodetect if None)
        file_memory_map (bool): Use memory mapping for large files (default: False)
        strict_match_size (bool): Strictly match nbt content size (default: True)

    Returns:
        CompoundTag or None if parsing fails
    """

def load_snbt(path: os.PathLike) -> Optional[CompoundTag]:
    """
    Parse CompoundTag from SNBT (String NBT) file

    Args:
        path (os.PathLike): Path to SNBT file

    Returns:
        CompoundTag or None if parsing fails
    """

def loads(
    content: Union[bytes, bytearray],
    format: Optional[NbtFileFormat] = None,
    strict_match_size: bool = True,
) -> Optional[CompoundTag]:
    """
    Parse CompoundTag from binary data

    Args:
        content (bytes): Binary NBT data
        format (NbtFileFormat, optional): Force specific format (autodetect if None)
        strict_match_size (bool): Strictly match nbt content size (default: True)

    Returns:
        CompoundTag or None if parsing fails
    """

def loads_base64(
    content: str,
    format: Optional[NbtFileFormat] = None,
    strict_match_size: bool = True,
) -> Optional[CompoundTag]:
    """
    Parse CompoundTag from Base64-encoded NBT

    Args:
        content (str): Base64-encoded NBT data
        format (NbtFileFormat, optional): Force specific format (autodetect if None)
        strict_match_size (bool): Strictly match nbt content size (default: True)

    Returns:
        CompoundTag or None if parsing fails
    """

def loads_snbt(
    content: str, parsed_length: Optional[int] = None
) -> Optional[CompoundTag]:
    """
    Parse CompoundTag from SNBT (String NBT) file

    Args:
        content (str): SNBT content

    Returns:
        CompoundTag or None if parsing fails
    """

def open(path: os.PathLike) -> Optional[NbtFile]:
    """
    Open a NBT file (auto detect)

    Args:
        path (os.PathLike): NBT file path

    Returns:
        Optional[NbtFile]: NbtFile or None if open failed
    """

def validate_content(
    content: Union[bytes, bytearray],
    format: NbtFileFormat = NbtFileFormat.LITTLE_ENDIAN,
    strict_match_size: bool = True,
) -> bool:
    """
    Validate NBT binary content

    Args:
        content (bytes): Binary data to validate
        format (NbtFileFormat): Expected format (default: LITTLE_ENDIAN)
        strict_match_size (bool): Strictly match nbt content size (default: True)

    Returns:
        bool: True if valid NBT, False otherwise
    """

def validate_file(
    path: os.PathLike,
    format: NbtFileFormat = NbtFileFormat.LITTLE_ENDIAN,
    file_memory_map: bool = False,
    strict_match_size: bool = True,
) -> bool:
    """
    Validate NBT file

    Args:
        path (os.PathLike): File path to validate
        format (NbtFileFormat): Expected format (default: LITTLE_ENDIAN)
        file_memory_map (bool): Use memory mapping (default: False)
        strict_match_size (bool): Strictly match nbt content size (default: True)

    Returns:
        bool: True if valid NBT file, False otherwise
    """



# Copyright © 2025 GlacieTeam.All rights reserved.
#
# This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy
# of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# SPDX-License-Identifier: MPL-2.0

from typing import overload, Union
from rapidnbt._NBT.tag import Tag
from rapidnbt._NBT.tag_type import TagType

class StringTag(Tag):
    """
    A tag contains a string
    """

    def __eq__(self, other: StringTag) -> bool:
        """
        Equality operator (==), case-sensitive comparison
        """

    def __getitem__(self, index: int) -> str:
        """
        Get character at specified position
        """

    def __hash__(self) -> int:
        """
        Compute hash value for Python hashing operations
        """

    @overload
    def __init__(self) -> None:
        """
        Construct an empty StringTag
        """

    @overload
    def __init__(self, str: str) -> None:
        """
        Construct from a Python string
        """

    @overload
    def __init__(self, str: Union[bytes, bytearray]) -> None:
        """
        Construct from a Python bytes / bytearray
        """

    def __len__(self) -> int:
        """
        Get the length of the string in bytes
        """

    def __repr__(self) -> str:
        """
        Official representation with quoted content
        """

    def __str__(self) -> str:
        """
        String representation (SNBT minimized format)
        """

    def copy(self) -> Tag:
        """
        Create a deep copy of this tag
        """

    def equals(self, other: Tag) -> bool:
        """
        Check if this tag equals another tag (same content and type)
        """

    def get_type(self) -> TagType:
        """
        Get the NBT type ID (String)
        """

    def hash(self) -> int:
        """
        Compute hash value of this tag (based on string content)
        """

    def load(self, stream: ...) -> None:
        """
        Load tag value from a binary stream (UTF-8)
        """

    def size(self) -> int:
        """
        Get the length of the string in bytes
        """

    def write(self, stream: ...) -> None:
        """
        Write tag to a binary stream (UTF-8 encoded)
        """

    def get(self) -> str:
        """
        Get the string content of this tag
        """

    def set(self, value: str) -> None:
        """
        Set the string content of this tag
        """

    @property
    def value(self) -> bytes:
        """
        Access the original string content of this tag
        """

    @value.setter
    def value(self, value: Union[bytes, bytearray, str]) -> None:
        """
        Access the original string content of this tag
        """



# Copyright © 2025 GlacieTeam.All rights reserved.
#
# This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy
# of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# SPDX-License-Identifier: MPL-2.0

from typing import overload, List, Dict, Union, Optional, Any
from rapidnbt._NBT.tag import Tag
from rapidnbt._NBT.tag_type import TagType
from rapidnbt._NBT.compound_tag_variant import CompoundTagVariant
from rapidnbt._NBT.byte_tag import ByteTag
from rapidnbt._NBT.short_tag import ShortTag
from rapidnbt._NBT.int_tag import IntTag
from rapidnbt._NBT.long_tag import LongTag
from rapidnbt._NBT.float_tag import FloatTag
from rapidnbt._NBT.double_tag import DoubleTag
from rapidnbt._NBT.byte_array_tag import ByteArrayTag
from rapidnbt._NBT.string_tag import StringTag
from rapidnbt._NBT.list_tag import ListTag
from rapidnbt._NBT.int_array_tag import IntArrayTag
from rapidnbt._NBT.long_array_tag import LongArrayTag

class CompoundTag(Tag):
    """
    A tag contains a tag compound
    """

    @staticmethod
    def from_binary_nbt(
        binary_data: Union[bytes, bytearray],
        little_endian: bool = True,
        header: bool = False,
    ) -> Optional[CompoundTag]:
        """
        Deserialize from binary NBT format
        """

    @staticmethod
    def from_network_nbt(
        binary_data: Union[bytes, bytearray],
    ) -> Optional[CompoundTag]:
        """
        Deserialize from Network NBT format
        """

    @staticmethod
    def from_snbt(
        snbt: str, parsed_length: Optional[int] = None
    ) -> Optional[CompoundTag]:
        """
        Parse from String NBT (SNBT) format
        """

    def __contains__(self, key: str) -> bool:
        """
        Check if key exists in the compound
        """

    def __delitem__(self, key: str) -> bool:
        """
        Remove key from the compound
        """

    def __eq__(self, other: Tag) -> bool:
        """
        Equality operator (==)
        """

    def __getitem__(self, key: str) -> CompoundTagVariant:
        """

        Get value by key (no exception, auto create if not found)
        """

    def __hash__(self) -> int:
        """
        Compute hash value for Python hashing operations
        """

    @overload
    def __init__(self) -> None:
        """
        Construct an empty CompoundTag
        """

    @overload
    def __init__(self, pairs: Dict[str, Any]) -> None:
        """
        Construct from a Dict[str, Any]
        Example:
            CompoundTag(["key1": 42, "key2": "value"])

        """

    def __iter__(self) -> List[str]:
        """
        Iterate over keys in the compound
        """

    def __len__(self) -> int:
        """

        Get number of key-value pairs
        """

    def __repr__(self) -> str:
        """
        Official string representation
        """

    def __setitem__(self, key: str, value: Any) -> None:
        """
        Set value by key
        """

    def __str__(self) -> str:
        """
        String representation (SNBT minimized format)
        """

    def clear(self) -> None:
        """
        Remove all elements from the compound
        """

    def clone(self) -> CompoundTag:
        """
        Create a deep copy of this compound tag
        """

    @overload
    def contains(self, key: str) -> bool:
        """
        Check if key exists
        """

    @overload
    def contains(self, key: str, type: TagType) -> bool:
        """
        Check if key exists and value type is the specific type
        """

    def copy(self) -> Tag:
        """
        Create a deep copy of this tag
        """

    def deserialize(self, stream: ...) -> None:
        """
        Deserialize compound from a binary stream
        """

    def empty(self) -> bool:
        """
        Check if the compound is empty
        """

    def equals(self, other: Tag) -> bool:
        """
        Check if this tag equals another tag
        """

    def get(self, key: str) -> CompoundTagVariant:
        """
        Get tag by key
        Throw KeyError if not found
        """

    def get_byte_tag(self, key: str) -> Optional[ByteTag]:
        """
        Get ByteTag
        Throw KeyError if not found or TypeError if wrong type
        """

    def get_byte_array_tag(self, key: str) -> Optional[ByteArrayTag]:
        """
        Get ByteArrayTag
        Throw KeyError if not found or TypeError if wrong type
        """

    def get_compound_tag(self, key: str) -> Optional[CompoundTag]:
        """
        Get CompoundTag
        Throw KeyError if not found or TypeError if wrong type
        """

    def get_double_tag(self, key: str) -> Optional[DoubleTag]:
        """
        Get DoubleTag
        Throw KeyError if not found or TypeError if wrong type
        """

    def get_float_tag(self, key: str) -> Optional[FloatTag]:
        """
        Get FloatTag value
        Throw KeyError if not found or TypeError if wrong type
        """

    def get_int_tag(self, key: str) -> Optional[IntTag]:
        """
        Get IntTag
        Throw KeyError if not found or TypeError if wrong type
        """

    def get_long_tag(self, key: str) -> Optional[LongTag]:
        """
        Get LongTag
        Throw KeyError if not found or TypeError if wrong type
        """

    def get_int_array_tag(self, key: str) -> Optional[IntArrayTag]:
        """
        Get IntArrayTag
        Throw KeyError if not found or TypeError if wrong type
        """

    def get_list_tag(self, key: str) -> Optional[ListTag]:
        """
        Get ListTag
        Throw KeyError if not found or TypeError if wrong type
        """

    def get_long_array_tag(self, key: str) -> Optional[LongArrayTag]:
        """
        Get LongArrayTag
        Throw KeyError if not found or TypeError if wrong type
        """

    def get_short_tag(self, key: str) -> Optional[ShortTag]:
        """
        Get ShortTag
        Throw KeyError if not found or TypeError if wrong type
        """

    def get_string_tag(self, key: str) -> Optional[StringTag]:
        """
        Get StringTag
        Throw KeyError if not found or TypeError if wrong type
        """

    def get_byte(self, key: str) -> Optional[int]:
        """
        Get byte value
        Throw KeyError if not found or TypeError if wrong type
        """

    def get_byte_array(self, key: str) -> Optional[bytes]:
        """
        Get byte array
        Throw KeyError if not found or TypeError if wrong type
        """

    def get_compound(self, key: str) -> Optional[Dict[str, CompoundTagVariant]]:
        """
        Get CompoundTag
        Throw KeyError if not found or TypeError if wrong type
        """

    def get_double(self, key: str) -> Optional[float]:
        """
        Get double value
        Throw KeyError if not found or TypeError if wrong type
        """

    def get_float(self, key: str) -> Optional[float]:
        """
        Get float value
        Throw KeyError if not found or TypeError if wrong type
        """

    def get_int(self, key: str) -> Optional[int]:
        """
        Get int value
        Throw KeyError if not found or TypeError if wrong type
        """

    def get_long(self, key: str) -> Optional[int]:
        """
        Get long value
        Throw KeyError if not found or TypeError if wrong type
        """

    def get_int_array(self, key: str) -> Optional[List[int]]:
        """
        Get int array
        Throw KeyError if not found or TypeError if wrong type
        """

    def get_list(self, key: str) -> Optional[List[CompoundTagVariant]]:
        """
        Get ListTag
        Throw KeyError if not found or TypeError if wrong type
        """

    def get_long_array(self, key: str) -> Optional[List[int]]:
        """
        Get long array
        Throw KeyError if not found or TypeError if wrong type
        """

    def get_short(self, key: str) -> Optional[int]:
        """
        Get short value
        Throw KeyError if not found or TypeError if wrong type
        """

    def get_string(self, key: str) -> Optional[str]:
        """
        Get string value
        Throw KeyError if not found or TypeError if wrong type
        """

    def get_type(self) -> TagType:
        """
        Get the NBT type ID (Compound)
        """

    def hash(self) -> int:
        """
        Compute hash value of this tag
        """

    def items(self) -> list:
        """
        Get list of (key, value) pairs in the compound
        """

    def keys(self) -> list:
        """
        Get list of all keys in the compound
        """

    def load(self, stream: ...) -> None:
        """
        Load compound from a binary stream
        """

    def merge(self, other: CompoundTag, merge_list: bool = False) -> None:
        """
        Merge another CompoundTag into this one

        Arguments:
            other: CompoundTag to merge from
            merge_list: If true, merge list contents instead of replacing
        """

    def pop(self, key: str) -> bool:
        """
        Remove key from the compound
        """

    def put(self, key: str, value: Any) -> None:
        """
        Put a value into the compound (automatically converted to appropriate tag type)
        """

    def put_byte(self, key: str, value: int) -> None:
        """
        Put a byte (uint8) value
        """

    def put_byte_array(self, key: str, value: Union[bytes, bytearray]) -> None:
        """
        Put a byte array (list of uint8)
        """

    def put_compound(self, key: str, value: Any) -> None:
        """
        Put a CompoundTag value (or dict that will be converted)
        """

    def put_double(self, key: str, value: float) -> None:
        """
        Put a double value
        """

    def put_float(self, key: str, value: float) -> None:
        """
        Put a float value
        """

    def put_int(self, key: str, value: int) -> None:
        """
        Put an int (int32) value
        """

    def put_long(self, key: str, value: int) -> None:
        """
        Put a long (int64) value
        """

    def put_int_array(self, key: str, value: List[int]) -> None:
        """
        Put an int array (list of int32)
        """

    def put_list(self, key: str, value: Any) -> None:
        """
        Put a ListTag value (or list/tuple that will be converted)
        """

    def put_long_array(self, key: str, value: List[int]) -> None:
        """
        Put a long array (list of int64)
        """

    def put_short(self, key: str, value: int) -> None:
        """
        Put a short (int16) value
        """

    def put_string(self, key: str, value: str) -> None:
        """
        Put a string value
        """

    def rename(self, old_key: str, new_key: str) -> bool:
        """
        Rename a key in the compound
        """

    def set(self, key: str, value: Any) -> None:
        """
        Set a value into the compound (automatically converted to appropriate tag type)
        """

    def set_byte(self, key: str, value: int) -> None:
        """
        Set a byte (uint8) value
        """

    def set_byte_array(self, key: str, value: Union[bytes, bytearray]) -> None:
        """
        Set a byte array (list of uint8)
        """

    def set_compound(self, key: str, value: Any) -> None:
        """
        Set a CompoundTag value (or dict that will be converted)
        """

    def set_double(self, key: str, value: float) -> None:
        """
        Set a double value
        """

    def set_float(self, key: str, value: float) -> None:
        """
        Set a float value
        """

    def set_int(self, key: str, value: int) -> None:
        """
        Set an int (int32) value
        """

    def set_long(self, key: str, value: int) -> None:
        """
        Set a long (int64) value
        """

    def set_int_array(self, key: str, value: List[int]) -> None:
        """
        Set an int array (list of int32)
        """

    def set_list(self, key: str, value: Any) -> None:
        """
        Set a ListTag value (or list/tuple that will be converted)
        """

    def set_long_array(self, key: str, value: List[int]) -> None:
        """
        Set a long array (list of int64)
        """

    def set_short(self, key: str, value: int) -> None:
        """
        Set a short (int16) value
        """

    def set_string(self, key: str, value: str) -> None:
        """
        Set a string value
        """

    def serialize(self, stream: ...) -> None:
        """
        Serialize compound to a binary stream
        """

    def size(self) -> int:
        """
        Get the size of the compound
        """

    def to_binary_nbt(self, little_endian: bool = True, header: bool = False) -> bytes:
        """
        Serialize to binary NBT format
        """

    def to_dict(self) -> dict:
        """
        Convert CompoundTag to a Python dictionary
        """

    def to_network_nbt(self) -> bytes:
        """
        Serialize to Network NBT format (used in Minecraft networking)
        """

    def values(self) -> list:
        """
        Get list of all values in the compound
        """

    def write(self, stream: ...) -> None:
        """
        Write compound to a binary stream
        """

    @property
    def value(self) -> Dict[str, CompoundTagVariant]:
        """
        Access the dict value of this tag
        """

    @value.setter
    def value(self, value: Dict[str, Any]) -> None:
        """
        Access the dict value of this tag
        """



# Copyright © 2025 GlacieTeam.All rights reserved.
#
# This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy
# of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# SPDX-License-Identifier: MPL-2.0

from typing import overload, List, Any, Dict, Union
from rapidnbt._NBT.snbt_format import SnbtFormat
from rapidnbt._NBT.tag_type import TagType
from rapidnbt._NBT.tag import Tag
from rapidnbt._NBT.end_tag import EndTag
from rapidnbt._NBT.byte_tag import ByteTag
from rapidnbt._NBT.short_tag import ShortTag
from rapidnbt._NBT.int_tag import IntTag
from rapidnbt._NBT.long_tag import LongTag
from rapidnbt._NBT.float_tag import FloatTag
from rapidnbt._NBT.double_tag import DoubleTag
from rapidnbt._NBT.byte_array_tag import ByteArrayTag
from rapidnbt._NBT.string_tag import StringTag
from rapidnbt._NBT.list_tag import ListTag
from rapidnbt._NBT.compound_tag import CompoundTag
from rapidnbt._NBT.int_array_tag import IntArrayTag
from rapidnbt._NBT.long_array_tag import LongArrayTag

TagVariant = Union[
    EndTag,
    ByteTag,
    ShortTag,
    IntTag,
    LongTag,
    FloatTag,
    DoubleTag,
    StringTag,
    ByteArrayTag,
    CompoundTag,
    ListTag,
    IntArrayTag,
    LongArrayTag,
]

class CompoundTagVariant:
    """
    A warpper of all tags, to provide morden API for NBT
    """

    def __contains__(self, index: str) -> bool:
        """
        Check if the value is in the CompoundTag.
        Throw TypeError is not hold a CompoundTag.
        """

    def __eq__(self, other: CompoundTagVariant) -> bool:
        """
        Check if this tag equals another tag"""

    def __float__(self) -> float:
        """
        Implicitly convert to float"""

    @overload
    def __getitem__(self, index: int) -> Tag:
        """
        Get value by object key
        """

    @overload
    def __getitem__(self, index: str) -> CompoundTagVariant:
        """
        Get value by array index
        """

    @overload
    def __init__(self) -> None:
        """
        Default Constructor
        """

    @overload
    def __init__(self, value: Any) -> None:
        """
        Construct from any Python object
        """

    def __int__(self) -> int:
        """
        Implicitly convert to int
        """

    def __iter__(self) -> List[CompoundTagVariant]:
        """
        Iterate over tags in the tag variant"""

    def __len__(self) -> int:
        """
        Get the size of the tag
        """

    def __repr__(self) -> str:
        """
        Official string representation
        """

    @overload
    def __setitem__(self, index: str, value: Any) -> None:
        """
        Set value by object key
        """

    @overload
    def __setitem__(self, index: int, value: Any) -> None:
        """
        Set value by array index
        """

    def __str__(self) -> str:
        """
        String representation (SNBT minimized format)
        """

    def append(self, value: Any) -> None:
        """
        Append a Tag element if self is ListTag
        Throw TypeError if wrong type
        """

    def assign(self, value: Any) -> None:
        """Assign value"""

    def as_byte_tag(self) -> ByteTag:
        """
        Convert to a ByteTag
        Throw TypeError if wrong type
        """

    def as_byte_array_tag(self) -> ByteArrayTag:
        """
        Convert to a ByteArrayTag
        Throw TypeError if wrong type
        """

    def as_compound_tag(self) -> CompoundTag:
        """
        Convert to a CompoundTag
        Throw TypeError if wrong type
        """

    def as_double_tag(self) -> DoubleTag:
        """
        Convert to a DoubleTag
        Throw TypeError if wrong type
        """

    def as_float_tag(self) -> FloatTag:
        """
        Convert to a FLoatTag
        Throw TypeError if wrong type
        """

    def as_int_tag(self) -> IntTag:
        """
        Convert to a IntTag
        Throw TypeError if wrong type
        """

    def as_long_tag(self) -> LongTag:
        """
        Convert to a LongTag
        Throw TypeError if wrong type
        """

    def as_int_array_tag(self) -> IntArrayTag:
        """
        Convert to a IntArrayTag
        Throw TypeError if wrong type
        """

    def as_list_tag(self) -> ListTag:
        """
        Convert to a ListTag
        Throw TypeError if wrong type
        """

    def as_long_array_tag(self) -> LongArrayTag:
        """
        Convert to a LongArrayTag
        Throw TypeError if wrong type
        """

    def as_short_tag(self) -> ShortTag:
        """
        Convert to a ShortTag
        Throw TypeError if wrong type
        """

    def as_string_tag(self) -> StringTag:
        """
        Convert to a StringTag
        Throw TypeError if wrong type
        """

    def get(self) -> TagVariant:
        """
        Get the tag variant
        """

    def get_byte(self) -> int:
        """
        Get the byte value
        Throw TypeError if wrong type
        """

    def get_bytes(self) -> bytes:
        """
        Get the original string value (bytes in StringTag)
        Throw TypeError if wrong type
        """

    def get_byte_array(self) -> bytes:
        """
        Get the byte array value
        Throw TypeError if wrong type
        """

    def get_compound(self) -> Dict[str, CompoundTagVariant]:
        """
        Get the CompoundTag as a dict value
        Throw TypeError if wrong type
        """

    def get_double(self) -> float:
        """
        Get the double value
        Throw TypeError if wrong type
        """

    def get_float(self) -> float:
        """
        Get the float value
        Throw TypeError if wrong type
        """

    def get_int(self) -> int:
        """
        Get the int value
        Throw TypeError if wrong type
        """

    def get_long(self) -> int:
        """
        Get the int64 value
        Throw TypeError if wrong type
        """

    def get_int_array(self) -> List[int]:
        """
        Get the int array value
        Throw TypeError if wrong type
        """

    def get_list(self) -> List[CompoundTagVariant]:
        """
        Get the ListTag as a list value
        Throw TypeError if wrong type
        """

    def get_long_array(self) -> List[int]:
        """
        Get the long array value
        Throw TypeError if wrong type
        """

    def get_short(self) -> int:
        """
        Get the short value
        Throw TypeError if wrong type
        """

    def get_string(self) -> str:
        """
        Get the string value
        Throw TypeError if wrong type
        """

    @overload
    def contains(self, index: str) -> bool:
        """
        Check if the value is in the CompoundTag.
        Throw TypeError is not hold a CompoundTag.
        """

    @overload
    def contains(self, key: str, type: TagType) -> bool:
        """
        Check if the value is in the CompoundTag and value type is the specific type.
        Throw TypeError is not hold a CompoundTag.
        """

    def clear(self) -> None:
        """
        Clear the data in the tag
        Throw TypeError if the tag can not be cleared.
        """

    def copy(self) -> Tag:
        """
        Create a deep copy of this tag
        """

    def get_type(self) -> TagType:
        """
        Get the NBT type ID
        """

    def hold(self, type: TagType) -> bool:
        """
        Check the NBT type ID
        """

    def is_array(self) -> bool:
        """
        Check whether the tag is a ListTag
        """

    def is_binary(self) -> bool:
        """
        Check whether the tag is a binary tag
        Example:
            ByteArrayTag, IntArrayTag, LongArrayTag
        """

    def is_boolean(self) -> bool:
        """
        Check whether the tag is a ByteTag
        """

    def is_null(self) -> bool:
        """
        Check whether the tag is an EndTag
        """

    def is_number(self) -> bool:
        """
        Check whether the tag is a number based tag
        Example:
            ByteTag, ShortTag, IntTag, LongTag, FloatTag, DoubleTag
        """

    def is_number_float(self) -> bool:
        """
        Check whether the tag is a float number based tag
        Example:
            FloatTag, DoubleTag
        """

    def is_number_integer(self) -> bool:
        """
        Check whether the tag is a integer number based tag
        Example:
            ByteTag, ShortTag, IntTag, LongTag
        """

    def is_object(self) -> bool:
        """
        Check whether the tag is a CompoundTag
        """

    def is_primitive(self) -> bool:
        """
        Check whether the tag is a primitive tag
        Example:
            ByteTag, ShortTag, IntTag, LongTag, FloatTag, DoubleTag, StringTag, ByteArrayTag, IntArrayTag, LongArrayTag
        """

    def is_string(self) -> bool:
        """
        Check whether the tag is a StringTag
        """

    def is_structured(self) -> bool:
        """
        Check whether the tag is a structured tag
        Example:
            CompoundTag, ListTag
        """

    def items(self) -> list:
        """
        Get list of (key, value) pairs in this tag
        Throw TypeError if wrong type
        """

    def merge(self, other: CompoundTagVariant, merge_list: bool = False) -> None:
        """
        Merge another CompoundTag into this one

        Arguments:
            other: CompoundTag to merge from
            merge_list: If true, merge list contents instead of replacing
        """

    @overload
    def pop(self, index: str) -> bool:
        """
        Remove key from the CompoundTag
        Throw TypeError if wrong type
        """

    @overload
    def pop(self, index: int) -> bool:
        """
        Remove key from the ListTag
        Throw TypeError if wrong type
        """

    def rename(self, index: str, new_name: str) -> bool:
        """
        Rename a key in the CompoundTag
        Throw TypeError if wrong type
        """

    def size(self) -> int:
        """
        Get the size of the tag
        """

    def to_json(self, indent: int = 4) -> str:
        """
        Convert tag to JSON string
        """

    def to_snbt(
        self, snbt_format: SnbtFormat = SnbtFormat.Default, indent: int = 4
    ) -> str:
        """
        Convert tag to SNBT string
        """

    @property
    def value(self) -> Any:
        """
        Access the tag value
        """

    @value.setter
    def value(self, value: Any) -> None:
        """
        Access the tag value
        """



# Copyright © 2025 GlacieTeam.All rights reserved.
#
# This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy
# of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# SPDX-License-Identifier: MPL-2.0

from typing import overload
from abc import ABC, abstractmethod
from rapidnbt._NBT.compound_tag_variant import CompoundTagVariant
from rapidnbt._NBT.snbt_format import SnbtFormat
from rapidnbt._NBT.tag_type import TagType
from rapidnbt._NBT.byte_tag import ByteTag
from rapidnbt._NBT.short_tag import ShortTag
from rapidnbt._NBT.int_tag import IntTag
from rapidnbt._NBT.long_tag import LongTag
from rapidnbt._NBT.float_tag import FloatTag
from rapidnbt._NBT.double_tag import DoubleTag
from rapidnbt._NBT.byte_array_tag import ByteArrayTag
from rapidnbt._NBT.string_tag import StringTag
from rapidnbt._NBT.list_tag import ListTag
from rapidnbt._NBT.compound_tag import CompoundTag
from rapidnbt._NBT.int_array_tag import IntArrayTag
from rapidnbt._NBT.long_array_tag import LongArrayTag

class Tag(ABC):
    """
    Base class for all NBT tags
    """

    @staticmethod
    def new_tag(type: TagType) -> Tag:
        """
        Create a new tag of the given type
        """

    def __eq__(self, other: Tag) -> bool:
        """
        Compare two tags for equality
        """

    @overload
    def __getitem__(self, index: int) -> Tag: ...
    @overload
    def __getitem__(self, key: str) -> CompoundTagVariant: ...
    def __hash__(self) -> int:
        """
        Compute hash value for Python hashing operations
        """

    def __repr__(self) -> str: ...
    def __str__(self) -> str: ...
    def as_byte_tag(self) -> ByteTag:
        """
        Convert to a ByteTag
        Throw TypeError if wrong type
        """

    def as_byte_array_tag(self) -> ByteArrayTag:
        """
        Convert to a ByteArrayTag
        Throw TypeError if wrong type
        """

    def as_compound_tag(self) -> CompoundTag:
        """
        Convert to a CompoundTag
        Throw TypeError if wrong type
        """

    def as_double_tag(self) -> DoubleTag:
        """
        Convert to a DoubleTag
        Throw TypeError if wrong type
        """

    def as_float_tag(self) -> FloatTag:
        """
        Convert to a FLoatTag
        Throw TypeError if wrong type
        """

    def as_int_tag(self) -> IntTag:
        """
        Convert to a IntTag
        Throw TypeError if wrong type
        """

    def as_long_tag(self) -> LongTag:
        """
        Convert to a LongTag
        Throw TypeError if wrong type
        """

    def as_int_array_tag(self) -> IntArrayTag:
        """
        Convert to a IntArrayTag
        Throw TypeError if wrong type
        """

    def as_list_tag(self) -> ListTag:
        """
        Convert to a ListTag
        Throw TypeError if wrong type
        """

    def as_long_array_tag(self) -> LongArrayTag:
        """
        Convert to a LongArrayTag
        Throw TypeError if wrong type
        """

    def as_short_tag(self) -> ShortTag:
        """
        Convert to a ShortTag
        Throw TypeError if wrong type
        """

    def as_string_tag(self) -> StringTag:
        """
        Convert to a StringTag
        Throw TypeError if wrong type
        """

    @abstractmethod
    def copy(self) -> Tag:
        """
        Create a deep copy of this tag
        """

    @abstractmethod
    def equals(self, other: Tag) -> bool:
        """
        Check if this tag equals another tag
        """

    @abstractmethod
    def get_type(self) -> TagType:
        """
        Get the type of this tag
        """

    @abstractmethod
    def hash(self) -> int:
        """
        Compute hash value of this tag
        """

    @abstractmethod
    def load(self, stream: ...) -> None:
        """
        Load tag from binary stream
        """

    @abstractmethod
    def write(self, stream: ...) -> None:
        """
        Write tag to binary stream
        """

    def to_json(self, indent: int = 4) -> str:
        """
        Convert tag to JSON string
        """

    def to_snbt(self, format: SnbtFormat = SnbtFormat.Default, indent: int = 4) -> str:
        """
        Convert tag to SNBT string
        """



# Copyright © 2025 GlacieTeam.All rights reserved.
#
# This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy
# of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# SPDX-License-Identifier: MPL-2.0

import os
from argparse import ArgumentParser
from pathlib import Path
from rapidnbt import CompoundTag, NbtCompressionType, NbtFileFormat, nbtio


def Info(output: str):  # pylint: disable=invalid-name
    print(f"\033[0m[INFO] {output}\033[0m")


def Warn(output: str):  # pylint: disable=invalid-name
    print(f"\033[33m[WARN] {output}\033[0m")


def Error(output: str):  # pylint: disable=invalid-name
    print(f"\033[31m[ERROR] {output}\033[0m")


def parse_file(path: str, fmt: NbtFileFormat):
    if fmt is not None:
        return nbtio.load(path, fmt)
    return nbtio.load_snbt(path)


def parse_args():
    parser = ArgumentParser(prog="nbt", description="Perform operations on nbt files.")

    # NBT File
    parser.add_argument("file", metavar="<file>", help="target NBT file")

    # Print Options
    parser.add_argument(
        "-p", "--print", action="store_true", help="print NBT as a formatted string"
    )
    parser.add_argument("-i", "--indent", type=int, default=4, help="NBT format indent")
    parser.add_argument(
        "-j", "--json", action="store_true", help="format NBT as a json string"
    )

    # Write Options
    parser.add_argument("-o", "--output", type=str, help="NBT output file path")

    outfmt = parser.add_mutually_exclusive_group()
    outfmt.add_argument(
        "--little",
        action="store_true",
        help="NBT output should use little endian format",
    )
    outfmt.add_argument(
        "--big",
        action="store_true",
        help="NBT output should use big endian format",
    )
    outfmt.add_argument(
        "--network",
        action="store_true",
        help="NBT output should use bedrock network format",
    )
    outfmt.add_argument(
        "--snbt",
        action="store_true",
        help="NBT output should use a formatted string nbt",
    )

    parser.add_argument(
        "--header",
        action="store_true",
        help="NBT output should write header",
    )
    parser.add_argument(
        "-m",
        "--merge",
        action="store_true",
        help="NBT output should merge exist NBT",
    )
    parser.add_argument(
        "--merge-list",
        action="store_true",
        help="NBT should merge ListTag instead of replace it",
    )
    parser.add_argument(
        "-c",
        "--compression",
        type=str,
        choices=["none", "gzip", "zlib"],
        default="gzip",
        help="NBT output should merge exist NBT",
    )

    return parser.parse_args()


def process_print_options(args, nbt: CompoundTag):
    if args.print is True:
        if args.json:
            print(nbt.to_json(args.indent))
        else:
            print(nbt.to_snbt(indent=args.indent))


def process_write_options(args, nbt: CompoundTag, fmt: NbtFileFormat, fp: str):
    if args.output is not None:
        if os.path.isabs(args.output):
            output = args.output
        else:
            output = os.path.join(os.getcwd(), args.output)
        os.makedirs(os.path.dirname(output), exist_ok=True)

        if args.snbt is False:
            if args.little is True:
                if args.header is False:
                    fmt = NbtFileFormat.LITTLE_ENDIAN
                else:
                    fmt = NbtFileFormat.LITTLE_ENDIAN_WITH_HEADER
            elif args.big is True:
                if args.big is False:
                    fmt = NbtFileFormat.BIG_ENDIAN
                else:
                    fmt = NbtFileFormat.BIG_ENDIAN_WITH_HEADER
            elif args.network:
                fmt = NbtFileFormat.BEDROCK_NETWORK

            compression = nbtio.detect_file_compression_type(fp)
            if args.compression == "gzip":
                compression = NbtCompressionType.GZIP
            elif args.compression == "zlib":
                compression = NbtCompressionType.ZLIB
            elif args.compression == "none":
                compression = NbtCompressionType.NONE

            if args.merge is False:
                nbtio.dump(nbt, output, fmt, compression)
            else:
                old = nbtio.load(output)
                if old is not None:
                    old.merge(nbt, args.merge_list)
                    nbtio.dump(old, output, fmt, compression)
                else:
                    Warn(
                        f"File {Path(output).absolute()} does not exist, skipping merge."
                    )
                    nbtio.dump(nbt, output, fmt, compression)

        else:
            if args.merge is False:
                nbtio.dump_snbt(nbt, output, indent=args.indent)
            else:
                old = nbtio.load_snbt(output)
                if old is not None:
                    old.merge(nbt, args.merge_list)
                    nbtio.dump_snbt(old, output, indent=args.indent)
                else:
                    Warn(
                        f"File {Path(output).absolute()} does not exist, skipping merge."
                    )
                    nbtio.dump_snbt(nbt, output, indent=args.indent)

        Info(f"NBT file generated at: {Path(output).absolute()}")


def main():
    args = parse_args()
    fp = args.file
    fmt = nbtio.detect_file_format(fp)
    nbt = parse_file(fp, fmt)
    if nbt is not None:
        process_print_options(args, nbt)
        process_write_options(args, nbt, fmt, fp)
    else:
        Error(f"No file exists to open: {fp}")



# Copyright © 2025 GlacieTeam.All rights reserved.
#
# This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy
# of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# SPDX-License-Identifier: MPL-2.0

from enum import Enum

class NbtFileFormat(Enum):
    """
    Enumeration of NBT binary file formats
    """

    LITTLE_ENDIAN = 0
    LITTLE_ENDIAN_WITH_HEADER = 1
    BIG_ENDIAN = 2
    BIG_ENDIAN_WITH_HEADER = 3
    BEDROCK_NETWORK = 4



# Copyright © 2025 GlacieTeam.All rights reserved.
#
# This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy
# of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# SPDX-License-Identifier: MPL-2.0

from typing import Any, Optional
from pathlib import Path
from rapidnbt._NBT.compound_tag import CompoundTag
from rapidnbt._NBT.nbt_compression_level import NbtCompressionLevel
from rapidnbt._NBT.nbt_compression_type import NbtCompressionType
from rapidnbt._NBT.nbt_file_format import NbtFileFormat
from rapidnbt._NBT.snbt_format import SnbtFormat

class NbtFile(CompoundTag):
    """
    NBT file
    Use nbtio.open() to open a NBT file.
    """

    def __enter__(self) -> NbtFile:
        """
        Enter context manager
        """

    def __exit__(self, exc_type: Any, exc_value: Any, traceback: Any) -> None:
        """
        Exit context manager
        """

    def __repr__(self) -> str:
        """
        Official string representation
        """

    def __str__(self) -> str:
        """
        String representation
        """

    def flush(self) -> None:
        """
        flush data to the file.
        """

    @property
    def compression_level(
        self,
    ) -> Optional[NbtCompressionLevel]:
        """
        File compression level
        """

    @compression_level.setter
    def compression_level(self, arg0: Optional[NbtCompressionLevel]) -> None: ...
    @property
    def compression_type(
        self,
    ) -> Optional[NbtCompressionType]:
        """
        File compression type
        """

    @compression_type.setter
    def compression_type(self, arg0: Optional[NbtCompressionType]) -> None:
        """
        File compression type
        """

    @property
    def file_format(self) -> Optional[NbtFileFormat]:
        """
        Binary file format
        """

    @file_format.setter
    def file_format(self, arg0: Optional[NbtFileFormat]) -> None:
        """
        Binary file format
        """

    @property
    def file_path(self) -> Path:
        """
        File path
        """

    @property
    def is_snbt(self) -> bool:
        """
        File is Snbt File
        """

    @is_snbt.setter
    def is_snbt(self, arg0: bool) -> None:
        """
        File is Snbt File
        """

    @property
    def snbt_format(self) -> Optional[SnbtFormat]:
        """
        File Snbt format
        """

    @snbt_format.setter
    def snbt_format(self, arg0: Optional[SnbtFormat]) -> None:
        """
        File Snbt format
        """

    @property
    def snbt_indent(self) -> Optional[int]:
        """
        File Snbt indent
        """

    @snbt_indent.setter
    def snbt_indent(self, arg0: Optional[int]) -> None:
        """
        File Snbt indent
        """



# Copyright © 2025 GlacieTeam.All rights reserved.
#
# This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy
# of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# SPDX-License-Identifier: MPL-2.0

from typing import overload, List, Union
from rapidnbt._NBT.tag import Tag
from rapidnbt._NBT.tag_type import TagType

class ByteArrayTag(Tag):
    """
    A tag contains a byte array
    """

    def __bytes__(self) -> bytes:
        """
        Convert to Python bytes object
        """

    def __eq__(self, other: Tag) -> bool:
        """
        Equality operator (==)
        """

    def __getitem__(self, index: int) -> int:
        """
        Get byte at specified index
        """

    def __hash__(self) -> int:
        """
        Compute hash value for Python hashing operations
        """

    @overload
    def __init__(self) -> None:
        """
        Construct an empty ByteArrayTag
        """

    @overload
    def __init__(self, arr: List[int]) -> None:
        """
        Construct from a list of bytes (e.g., [1, 2, 3])
        """

    def __iter__(self) -> List[int]:
        """
        Iterate over bytes in the array
        """

    def __len__(self) -> int:
        """
        Get number of bytes in the array
        """

    def __repr__(self) -> str:
        """
        Official string representation
        """

    def __setitem__(self, index: int, value: int) -> None:
        """
        Set byte at specified index
        """

    def __str__(self) -> str:
        """
        String representation (SNBT minimized format)
        """

    def append(self, value: int) -> None:
        """
        Add a byte to the end of the array
        """

    def assign(self, bytes: Union[bytes, bytearray]) -> ByteArrayTag:
        """
        Assign new binary data from a list of bytes
        """

    def clear(self) -> None:
        """
        Clear all byte data
        """

    def copy(self) -> Tag:
        """
        Create a deep copy of this tag
        """

    def data(self) -> memoryview:
        """
        Get a raw memory view of the byte data
        """

    def equals(self, other: Tag) -> bool:
        """
        Check if this tag equals another tag (same bytes and type)
        """

    def get_type(self) -> TagType:
        """
        Get the NBT type ID (ByteArray)
        """

    def hash(self) -> int:
        """
        Compute hash value of this tag
        """

    def load(self, stream: ...) -> None:
        """
        Load tag value from a binary stream
        """

    @overload
    def pop(self, index: int) -> bool:
        """
        Remove byte at specified index
        """

    @overload
    def pop(self, start_index: int, end_index: int) -> bool:
        """
        Remove bytes in the range [start_index, end_index)
        """

    def reserve(self, size: int) -> None:
        """
        Preallocate memory for future additions
        """

    def size(self) -> int:
        """
        Get number of bytes in the array
        """

    def write(self, stream: ...) -> None:
        """
        Write tag to a binary stream
        """

    @property
    def value(self) -> bytes:
        """
        Access the byte array as a list of integers (0-255)
        """

    @value.setter
    def value(self, value: Union[bytes, bytearray]) -> None:
        """
        Access the byte array as a list of integers (0-255)
        """



# Copyright © 2025 GlacieTeam.All rights reserved.
#
# This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy
# of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# SPDX-License-Identifier: MPL-2.0

from rapidnbt.cli import main

if __name__ == "__main__":
    main()



# Copyright © 2025 GlacieTeam.All rights reserved.
#
# This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy
# of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# SPDX-License-Identifier: MPL-2.0

"""Python bindings for NBT library"""

from rapidnbt._NBT.tag_type import TagType
from rapidnbt._NBT.snbt_format import SnbtFormat
from rapidnbt._NBT.compound_tag_variant import CompoundTagVariant
from rapidnbt._NBT.tag import Tag
from rapidnbt._NBT.end_tag import EndTag
from rapidnbt._NBT.byte_tag import ByteTag
from rapidnbt._NBT.short_tag import ShortTag
from rapidnbt._NBT.int_tag import IntTag
from rapidnbt._NBT.long_tag import LongTag
from rapidnbt._NBT.float_tag import FloatTag
from rapidnbt._NBT.double_tag import DoubleTag
from rapidnbt._NBT.byte_array_tag import ByteArrayTag
from rapidnbt._NBT.string_tag import StringTag
from rapidnbt._NBT.list_tag import ListTag
from rapidnbt._NBT.compound_tag import CompoundTag
from rapidnbt._NBT.int_array_tag import IntArrayTag
from rapidnbt._NBT.long_array_tag import LongArrayTag
from rapidnbt._NBT.nbt_file_format import NbtFileFormat
from rapidnbt._NBT.nbt_compression_level import NbtCompressionLevel
from rapidnbt._NBT.nbt_compression_type import NbtCompressionType
from rapidnbt._NBT.nbt_file import NbtFile
from rapidnbt._NBT import nbtio

__all__ = [
    "TagType",
    "SnbtFormat",
    "CompoundTagVariant",
    "Tag",
    "EndTag",
    "ByteTag",
    "ShortTag",
    "IntTag",
    "LongTag",
    "FloatTag",
    "DoubleTag",
    "ByteArrayTag",
    "StringTag",
    "ListTag",
    "CompoundTag",
    "IntArrayTag",
    "LongArrayTag",
    "nbtio",
    "NbtCompressionLevel",
    "NbtCompressionType",
    "NbtFileFormat",
    "NbtFile",
]



# Copyright © 2025 GlacieTeam.All rights reserved.
#
# This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy
# of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# SPDX-License-Identifier: MPL-2.0

from enum import Enum

class TagType(Enum):
    """
    The tag type enum
    """

    End = 0
    Byte = 1
    Short = 2
    Int = 3
    Long = 4
    Float = 5
    Double = 6
    ByteArray = 7
    String = 8
    List = 9
    Compound = 10
    IntArray = 11
    LongArray = 12



# Copyright © 2025 GlacieTeam.All rights reserved.
#
# This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy
# of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# SPDX-License-Identifier: MPL-2.0

from typing import overload, List, Any
from rapidnbt._NBT.compound_tag_variant import CompoundTagVariant
from rapidnbt._NBT.tag import Tag
from rapidnbt._NBT.tag_type import TagType

class ListTag(Tag):
    """
    A tag contains a tag list
    """

    def __eq__(self, other: Tag) -> bool:
        """
        Equality operator (==)
        """

    def __getitem__(self, index: int) -> Tag:
        """
        Get element at specified index
        """

    def __hash__(self) -> int:
        """
        Compute hash value for Python hashing operations
        """

    @overload
    def __init__(self) -> None:
        """
        Construct an empty ListTag
        """

    @overload
    def __init__(self, elements: List[Any]) -> None:
        """
        Construct from a list of Tag elements (e.g., [IntTag(1), StringTag('test')])
        """

    def __iter__(self) -> List[Tag]:
        """
        Iterate over elements in the list
        """

    def __len__(self) -> int:
        """
        Get number of elements in the list
        """

    def __repr__(self) -> str:
        """
        Official string representation
        """

    def __setitem__(self, index: int, element: Any) -> None:
        """
        Set element at specified index
        """

    def __str__(self) -> str:
        """
        String representation (SNBT minimized format)
        """

    def append(self, element: Any) -> None:
        """
        Append a Tag element to the list
        """

    def clear(self) -> None:
        """
        Remove all elements from the list
        """

    def copy(self) -> Tag:
        """
        Create a deep copy of this tag
        """

    def copy_list(self) -> ListTag:
        """
        Create a deep copy of this list (same as copy() but returns ListTag)
        """

    def empty(self) -> bool:
        """
        Check if the list is empty
        """

    def equals(self, other: Tag) -> bool:
        """
        Check if this tag equals another tag (same elements in same order)
        """

    def get_element_type(self) -> TagType:
        """
        Get the type of elements in this list (returns nbt.Type enum)
        """

    def get_type(self) -> TagType:
        """
        Get the NBT type ID (List)
        """

    def hash(self) -> int:
        """
        Compute hash value of this tag
        """

    def insert(self, index: int, element: Any) -> None:
        """
        Insert element at specified position
        """

    def load(self, stream: ...) -> None:
        """
        Load list from a binary stream
        """

    def merge(self, other: ListTag) -> None:
        """
        Merge another ListTag into this one (appends all elements)
        """

    def get_list(self) -> List[CompoundTagVariant]:
        """
        Convert ListTag to a Python list
        """

    @overload
    def pop(self, index: int) -> bool:
        """
        Remove element at specified index
        """

    @overload
    def pop(self, start_index: int, end_index: int) -> bool:
        """
        Remove elements in the range [start_index, end_index)
        """

    def reserve(self, size: int) -> None:
        """
        Preallocate memory for future additions
        """

    def size(self) -> int:
        """
        Get number of elements in the list
        """

    def write(self, stream: ...) -> None:
        """
        Write list to a binary stream
        """

    @property
    def value(self) -> List[CompoundTagVariant]:
        """
        Access the list value of this tag
        """

    @value.setter
    def value(self, value: List[Any]) -> None:
        """
        Access the list value of this tag
        """



# Copyright © 2025 GlacieTeam.All rights reserved.
#
# This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy
# of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# SPDX-License-Identifier: MPL-2.0

from enum import IntFlag

class SnbtFormat(IntFlag):
    """
    The SNBT format enum
    You can use | operation to combime flags
    Example:
        format = SnbtFormat.Classic | SnbtFormat.ForceUppercase
    """

    Minimize = 0
    CompoundLineFeed = 1
    ListArrayLineFeed = 2
    PrettyFilePrint = 3
    Default = 3
    BinaryArrayLineFeed = 4
    ArrayLineFeed = 6
    AlwaysLineFeed = 7
    ForceLineFeedIgnoreIndent = 8
    ForceAscii = 16
    ForceQuote = 32
    Classic = 35
    ForceUppercase = 64
    MarkIntTag = 128
    MarkDoubleTag = 256
    MarkExtra = 384
    CommentMarks = 512
    Jsonify = 551



import os
import shutil
import subprocess
from setuptools import Distribution, setup
from setuptools.command.build_ext import build_ext
from setuptools.command.build_py import build_py

PACKAGE_NAME = "rapidnbt"
EXTENSION_FILENAME = "_NBT"


class XMakeBuild(build_ext):
    def _clean(self):
        for root, _, files in os.walk(f"./{PACKAGE_NAME}"):
            for file_name in files:
                file_path = os.path.join(root, file_name)
                if file_path.endswith((".so", ".pyd")):
                    os.remove(file_path)

    def _copy_binary(self):
        shutil.copy(
            f"./build/bin/{EXTENSION_FILENAME}",
            os.path.join(
                f"./{PACKAGE_NAME}", self.get_ext_filename(EXTENSION_FILENAME)
            ),
        )

    def run(self):
        self._clean()
        subprocess.run(["xmake", "f", "--mode=release", "-y", "--root"], check=True)
        subprocess.run(["xmake", "--all", "-y", "--root"], check=True)
        self._copy_binary()


class XmakeDistribution(Distribution):
    def has_ext_modules(self):
        return True


class XmakeCommand(build_py):
    def run(self):
        self.run_command("build_ext")
        super().run()


setup(
    cmdclass={
        "build_ext": XMakeBuild,
        "build_py": XmakeCommand,
    },
    distclass=XmakeDistribution,
)



# Copyright © 2025 GlacieTeam.All rights reserved.
#
# This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy
# of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# SPDX-License-Identifier: MPL-2.0

from typing import overload
from rapidnbt._NBT.tag import Tag
from rapidnbt._NBT.tag_type import TagType

class FloatTag(Tag):
    """
    A tag contains a float
    """

    def __eq__(self, other: Tag) -> bool:
        """
        Equality operator (==)
        """

    def __float__(self) -> float:
        """
        Convert to Python float (for float(tag) operations)
        """

    def __hash__(self) -> int:
        """
        Compute hash value for Python hashing operations
        """

    @overload
    def __init__(self) -> None:
        """
        Construct a FloatTag with default value (0.0)
        """

    @overload
    def __init__(self, value: float) -> None:
        """
        Construct a FloatTag from a floating-point value
        """

    def __repr__(self) -> str:
        """
        Official string representation including type information
        """

    def __str__(self) -> str:
        """
        String representation (SNBT minimized format)
        """

    def assign(self, value: float) -> FloatTag:
        """
        Assign a new floating-point value to this tag
        """

    def copy(self) -> Tag:
        """
        Create a deep copy of this tag
        """

    def equals(self, other: Tag) -> bool:
        """
        Check if this tag equals another tag
        """

    def get_type(self) -> TagType:
        """
        Get the NBT type ID (Float)
        """

    def hash(self) -> int:
        """
        Compute hash value of this tag
        """

    def load(self, stream: ...) -> None:
        """
        Load tag value from a binary stream
        """

    def write(self, stream: ...) -> None:
        """
        Write tag to a binary stream
        """

    @property
    def value(self) -> float:
        """
        Access the floating-point value of this tag
        """

    @value.setter
    def value(self, value: float) -> None:
        """
        Access the floating-point value of this tag
        """



# Copyright © 2025 GlacieTeam.All rights reserved.
#
# This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy
# of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# SPDX-License-Identifier: MPL-2.0

from enum import Enum

class NbtCompressionLevel(Enum):
    """
    Enumeration of compression levels
    """

    DEFAULT = -1
    NO_COMPRESSION = 0
    BEST_SPEED = 1
    LOW = 2
    MEDIUM_LOW = 3
    MEDIUM = 4
    MEDIUM_HIGH = 5
    HIGH = 6
    VERY_HIGH = 7
    ULTRA = 8
    BEST_COMPRESSION = 9



# Copyright © 2025 GlacieTeam.All rights reserved.
#
# This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy
# of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# SPDX-License-Identifier: MPL-2.0

from typing import overload, List
from rapidnbt._NBT.tag import Tag
from rapidnbt._NBT.tag_type import TagType

class IntArrayTag(Tag):
    """
    A tag contains an int array
    """

    def __hash__(self) -> int:
        """
        Compute hash value for Python hashing operations
        """

    def __contains__(self, value: int) -> bool:
        """
        Check if value is in the array
        """

    def __eq__(self, other: IntArrayTag) -> bool:
        """
        Equality operator (==)
        """

    def __getitem__(self, index: int) -> int:
        """
        Get element at index without bounds checking
        """

    @overload
    def __init__(self) -> None:
        """
        Construct an empty IntArrayTag
        """

    @overload
    def __init__(self, values: List[int]) -> None:
        """
        Construct from a list of integers
        Example:
            IntArrayTag([1, 2, 3])
        """

    def __iter__(self) -> List[int]:
        """
        Iterate over elements in the array
        Example:
            for value in int_array:
                print(value)
        """

    def __len__(self) -> int:
        """
        Get number of int in the array
        """

    def __repr__(self) -> str:
        """
        Official string representation
        """

    def __setitem__(self, index: int, value: int) -> None:
        """
        Set element at index
        """

    def __str__(self) -> str:
        """
        String representation (SNBT minimized format)
        """

    def append(self, value: int) -> None:
        """
        Append an integer to the end of the array
        """

    def assign(self, values: List[int]) -> IntArrayTag:
        """
        Assign new values to the array
        Returns the modified array
        """

    def clear(self) -> None:
        """
        Remove all elements from the array
        """

    def copy(self) -> Tag:
        """
        Create a deep copy of this tag
        """

    def empty(self) -> bool:
        """
        Check if the array is empty
        """

    def equals(self, other: Tag) -> bool:
        """
        Check if this tag equals another tag
        """

    def get_type(self) -> TagType:
        """
        Get the NBT type ID (int array)
        """

    def hash(self) -> int:
        """
        Compute hash value of this tag
        """

    def load(self, stream: ...) -> None:
        """
        Load int array from a binary stream
        """

    @overload
    def pop(self, index: int) -> bool:
        """
        Remove element at specified index
        Returns:
            True if successful, False if index out of range
        """

    @overload
    def pop(self, start_index: int, end_index: int) -> bool:
        """
        Remove elements in the range [start_index, end_index)

        Arguments:
            start_index: First index to remove (inclusive)
            end_index: End index (exclusive)

        Returns:
            True if successful, False if indices out of range
        """

    def reserve(self, capacity: int) -> None:
        """
        Reserve storage capacity for the array

        Arguments:
            capacity: Minimum capacity to reserv)
        """

    def size(self) -> int:
        """
        Get number of elements in the array
        """

    def write(self, stream: ...) -> None:
        """
        Write int array to a binary stream
        """

    @property
    def value(self) -> List[int]:
        """
        Access the int array as a list of integers
        """

    @value.setter
    def value(self, value: List[int]) -> None:
        """
        Access the int array as a list of integers
        """
